%Author : Ming-Hen Tsai and Chia-Hua Ho on Apr 18 2010

%Parameters:
%querymethod : 1 -> all, 2->random, 3-> simple , 4->uniform, 5->asin, 7->chebyshev
%sp : number of points initially. Maybe 8 is good for all experiments
%solver_type : 1 -> libsvm, 2-> liblinear
%solver_s : -s parameter in libsvm/liblinear ...

%Notes:
%Best type of one-class solvers are encoded in this code.
function [ALC] = run_mixsvm(dataname,querymethod,sp,solver_type,solver_s)

%datanames
correctdata = ['../data/' dataname '.correct'];
traindata = ['../data/' dataname '.train'];
osvmsvrdata = ['../data/' dataname '.osvmsvr'];
trainmodel = [dataname '.train.model'];
osvmsvrmodel = [dataname '.osvmsvr.model'];
neg1data = ['../data/' dataname '.neg1'];
neg1model = [dataname '.neg1.model'];

%learner names : 
libsvminf = '../classifier/libsvm-infensemble-2.8-fv/';
libsvm = '../classifier/libsvm-2.9-fv/';
liblinear = '../classifier/liblinear-1.51-fv/';

%The first positive data
onepos = 1;
if strcmp(dataname,'hiva');
	onepos = 1;
	svmparam = ' -c 2 ';
	osvmsvrmap = '-t 0';
	posratio = 0.0352;
elseif strcmp(dataname,'ibn_sina');
	onepos = 4;	
	osvmsvrmap = '-t 2';
	svmparam = ' -c 8 -g 0.03125 ';
	posratio = 0.3784;
elseif strcmp(dataname,'nova');
	onepos = 11;	
	osvmsvrmap = '-t 0';
	svmparam = ' -c 32 ';
	posratio = 0.2845;
elseif strcmp(dataname,'orange');
	onepos = 54;	
	osvmsvrmap = '-t 2';
	svmparam = ' -c 0.125 -w1 4 ';
	posratio = 0.0178;
elseif strcmp(dataname,'sylvaexp');
	onepos = 4;	
	osvmsvrmap = '-t 0';
	posratio = 0.0615;
elseif strcmp(dataname,'sylva');
	onepos = 4;	
	osvmsvrmap = '-t 2';
	posratio = 0.0615;
	svmparam = ' ';
elseif strcmp(dataname,'zebra');
	onepos = 23;	
	osvmsvrmap = '-t 2';
	posratio = 0.0458;
	svmparam = ' -c 32 -g 0.5 ';
end

%data processing
[y x] = libsvmread(correctdata);
y(:) = -1;
y(onepos) = 1;
l = length(y); % Legnth of all data
libsvmwrite(neg1data,y,x);
libsvmwrite(traindata,[1],x(onepos,:));

%Truncate queried.sample
fp = fopen('queried.sample','w');
fprintf(fp,'%d\n',onepos);
fclose(fp);

over = 0;
sequence = sp.*[2.^(0:100)];
for j=1:length(sequence)
	if j == 1
%		[Yc,X] = libsvmread(correctdata);
%		l = length(Yc);
%		Y(1:l) = 0;
%		Y(onepos) = 1;

		if strcmp(dataname,'hiva') || strcmp(dataname,'nova')
			system([liblinear 'train -w-1 1e-11 -s 0 '  neg1data]);
			system([liblinear 'predict ' correctdata ' ' neg1model ' '  dataname '.predict']);
		else
		%svmlin r = 0.1
			y(:) = 0;
			y(onepos) = 1;
			[w fv] = svmlin('-A 2 -R 0.1',x,y);

			fp = fopen([dataname '.predict'],'w');
			fprintf(fp,'%.16e\n',fv);
			fclose(fp);
		end

	else
		disp(['In ' dataname '.predict']);
		disp(solver_type);
		if solver_type == 1
			system([libsvm 'svm-train -m 2000 ' svmparam ' ' traindata]);
			system([libsvm 'svm-predict ' correctdata ' ' trainmodel ' '  dataname '.predict']);
		elseif solver_type == 2
			param = sprintf(' -s %d ',solver_s);
			system([liblinear 'train ' svmparam ' ' param ' ' traindata]);
			system([liblinear 'predict ' correctdata ' ' trainmodel ' '  dataname '.predict']);
			disp([dataname '.predict']);
		end
	end

	if j == 15
		idxs = all_query(dataname,sequence(j),floor(l)) ;
	else
	if querymethod == 1
		idxs = all_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 2
		idxs = rand_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 3
		idxs = simple_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 4
		idxs = equi_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 5
		idxs = full_asin_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 6
		idxs = asin_query(dataname,sequence(j),floor(l)) ;
	elseif querymethod == 7
		idxs = cheby_query(dataname,sequence(j),floor(l));
	end
	end

	%%DO QUERY NOW
	%idxs = rand_query(2^j,floor(l/2));	
	%length(idxs);
	%File is written after query. 
	%We do this because we can thus submit easily.

	try
		qsample = load('queried.sample');
	catch
		qsample = [];
	end

	try
		%Load query
		qsample_toadd = load([dataname '.sample']);
		qsample_toadd = setdiff(qsample_toadd, qsample);
		qsample = union(qsample,qsample_toadd);
	catch
		over = 1;
		qsample_toadd = [];
	end

	%Write AUC
	auc = count_auc(dataname);
	fp = fopen([dataname '.auc'],'a');
	fprintf(fp,'%.16g %d\n',auc,length(qsample_toadd));
	fclose(fp);

	%system('./procceed.py');
	%disp('SUBMITTED');
	if(over == 1) break; end

	%Write Svm Data
	[labels data] = libsvmread(correctdata);
	libsvmwrite(traindata,labels(qsample),data(qsample,:));
	

	%Write queried File
	fp = fopen('queried.sample','w');
	fprintf(fp,'%d\n',qsample);
	fclose(fp);

end

fpp = popen(['./count_GAUC.py ' dataname '.auc'],'r');
ALC = fscanf(fpp,'%g');
fclose(fpp);

%return ;
system(['rm -f ' 'queried.sample']);
system(['rm -f ' dataname '.*']);
system(['rm -f ' traindata]);
system(['rm -f ' trainmodel]);
system(['rm -f ' osvmsvrdata]);
system(['rm -f ' osvmsvrmodel]);
system(['rm -f ' neg1data]);
system(['rm -f ' neg1model]);
system(['rm -f *.predict']);
